
#define BLYNK_TEMPLATE_ID           "TMPL6iizF8DzE"
#define BLYNK_TEMPLATE_NAME         "Quickstart Device"
#define BLYNK_AUTH_TOKEN            "Gu8AKmJC7lxmWe29YkVmhP4z0YwLXIsY"



#include <WiFi.h>
#include <WiFiClient.h>
#include <BlynkSimpleEsp32.h>
#define DHTPIN 4
#define DHTTYPE DHT22
#define NOISE_DETECT 35
#define TONE_OUTPUT_PIN 26
const int TONE_PWM_CHANNEL = 0;


#include "pitches.h"
#include <DHT.h>
#include <MPU9250_asukiaaa.h>

MPU9250_asukiaaa movementSensor;
float gXPrev, gYPrev, gZPrev;
const float threshold = 17;
unsigned long lastMovementTime = 0; // Variable to store the last movement time
unsigned long lastTempTime = 0;
bool sleepingSessionStarted = false;
float prevMovement = 0;
float curMovement = 0;
bool wakeBabyUp = false;

DHT dht(DHTPIN, DHTTYPE);

// The melody that will be played on demand
int melody[] = {
  NOTE_E5, NOTE_DS5, NOTE_E5, NOTE_DS5, NOTE_E5, NOTE_B4, NOTE_D5, NOTE_C5, NOTE_A4,
  NOTE_C4, NOTE_E4, NOTE_A4, NOTE_B4, NOTE_E4, NOTE_GS4, NOTE_B4, NOTE_C5,
  NOTE_E4, NOTE_E5, NOTE_DS5, NOTE_E5, NOTE_DS5, NOTE_E5, NOTE_B4, NOTE_D5, NOTE_C5, NOTE_A4,
  NOTE_C4, NOTE_E4, NOTE_A4, NOTE_B4, NOTE_E4, NOTE_C5, NOTE_B4, NOTE_A4
};


int noteDurations[] = {
  6, 6, 6, 6, 6, 6, 6, 6, 2,
  6, 6, 6, 2, 6, 6, 6, 2,
  6, 6, 6, 6, 6, 6, 6, 6, 6, 2,
  6, 6, 6, 2, 6, 6, 6, 1,
};


// WiFi credentials.
char ssid[] = "Burgerpizz";
char pass[] = "16161212";

BlynkTimer timer;

// This function is called every time the Virtual Pin 0 state changes
BLYNK_WRITE(V0)
{
  Blynk.virtualWrite(V2, dht.readTemperature()); 
  int value = param.asInt();
  if (value == 1) {
    sleepingSessionStarted = true;
  } else {
    sleepingSessionStarted = false;
    Blynk.virtualWrite(V1, 0); 
    Blynk.virtualWrite(V2, 0); 
    Blynk.virtualWrite(V3, 0); 
    Blynk.virtualWrite(V4, 0); 
  }
}

BLYNK_WRITE(V3)
{
  // setting a variable that stores the incoming LED switch state (1 or 0)
  int value = param.asInt();
  Serial.println("V3 value is:");
  Serial.println(value);

  if (value > 0) {
    wakeBabyUp = true;
    Serial.println("Baby is up");
  } else {
    wakeBabyUp = false;
  }
}


// Function to print time in hh:mm:ss format
void printTime(unsigned long millisecs) {
  unsigned long seconds = millisecs / 1000;
  unsigned long minutes = seconds / 60;
  unsigned long hours = minutes / 60;
  seconds %= 60;
  minutes %= 60;
  hours %= 24;
  if (hours < 10) Serial.print("0");
  Serial.print(hours);
  Serial.print(":");
  if (minutes < 10) Serial.print("0");
  Serial.print(minutes);
  Serial.print(":");
  if (seconds < 10) Serial.print("0");
  Serial.print(seconds);
}

// This function sends Arduino's uptime every second to Virtual Pin 2.
void myTimerEvent()
{
  movementSensor.gyroUpdate();

  if(sleepingSessionStarted) {
      float gX = movementSensor.gyroX();
      float gY = movementSensor.gyroY();
      float gZ = movementSensor.gyroZ();
    
      // Check if there is a significant change in gyroscope readings and write it to the analyzer
      unsigned long currentTime = millis();
      if (fabs(gX - gXPrev) > threshold || fabs(gY - gYPrev) > threshold || fabs(gZ - gZPrev) > threshold) {
        if (currentTime - lastMovementTime > 1000) { // Print only if a second has passed since the last movement
          Serial.print("Movement detected at ");
          printTime(currentTime); // Print current time
          Serial.print(" - gX: ");
          Serial.print(gX);
          Serial.print("\t gY: ");
          Serial.print(gY);
          Serial.print("\t gZ: ");
          Serial.println(gZ);
          curMovement = sqrt(pow(gX - gXPrev, 2) +
                                         pow(gY - gYPrev, 2) +
                                         pow(gZ - gZPrev, 2))/100;
          Blynk.virtualWrite(V1, curMovement);  
          Serial.println(curMovement);
          prevMovement = curMovement;                      
          lastMovementTime = currentTime; // Update last movement time
        }
      }
      Blynk.virtualWrite(V1, 0);

      // Measure temperature every minute and write it to the analyzer
      if(currentTime - lastTempTime > 60000) {
        float temperature = dht.readTemperature();
        Serial.println(F("%  Temperature: "));
        Serial.println(temperature);
        Blynk.virtualWrite(V2, temperature); 
        lastTempTime = currentTime;
      }

      // Detect noises and write it to the analyzer
      float noiseValue = (analogRead(NOISE_DETECT));
      Serial.println("Noise level: ");
      Serial.println(noiseValue);
      if(noiseValue > 1800) {
        Blynk.virtualWrite(V4, noiseValue); 
        Serial.println("Noise level: ");
        Serial.println(noiseValue);
      }
      Blynk.virtualWrite(V4, 0); 

       // Play music if wake up botton is pressed
      if(wakeBabyUp) {
          for (int thisNote = 0; thisNote < sizeof(melody)/sizeof(int); thisNote++) {
            int noteDuration = (1000 / noteDurations[thisNote])*0.8;
            ledcWriteTone(TONE_PWM_CHANNEL, melody[thisNote]);
            delay(noteDuration);
            ledcWrite(TONE_PWM_CHANNEL, 0);
            int pauseBetweenNotes = noteDuration * 0.20;
            delay(pauseBetweenNotes);
          }
        wakeBabyUp = false;
        Blynk.virtualWrite(V3, 0);
          
      }
  }

}

void setup()
{
  Serial.begin(115200);

  Blynk.begin(BLYNK_AUTH_TOKEN, ssid, pass);
  Wire.begin();
  
  
  // Activate movement sensor
  movementSensor.setWire(&Wire);
  movementSensor.beginGyro();
  
  // Activate temperature sensor
  dht.begin();
  ledcAttachPin(TONE_OUTPUT_PIN, TONE_PWM_CHANNEL);

  // Setup a function to be called every second to avoid overflow
  timer.setInterval(1000L, myTimerEvent);
}

void loop()
{
  Blynk.run();
  timer.run();
}
